@@ -12,7 +12,7 @@ Copyright: 2021 Martins Bruveris
 Copyright: 2021 Rishigami
 """
 from dataclasses import dataclass
-from typing import Tuple
+from typing import List, Tuple
 
 import numpy as np
 import tensorflow as tf
@@ -28,7 +28,7 @@ __all__ = ["SwinTransformer", "SwinTransformerConfig"]
 @dataclass
 class SwinTransformerConfig(ModelConfig):
     nb_classes: int = 1000
-    in_chans: int = 3
+    in_channels: int = 3
     input_size: Tuple[int, int] = (224, 224)
     patch_size: int = 4
     embed_dim: int = 96
@@ -44,7 +44,6 @@ class SwinTransformerConfig(ModelConfig):
     # Other parameters
     norm_layer: str = "layer_norm"
     act_layer: str = "gelu"
-    ape: bool = False  # Absolute position embedding
     patch_norm: bool = True
     # Parameters for inference
     interpolate_input: bool = False
@@ -186,7 +185,7 @@ class WindowAttention(tf.keras.layers.Layer):
 
         nw = mask.get_shape()[0]  # tf.shape(mask)[0]
         mask = tf.expand_dims(tf.expand_dims(mask, axis=1), axis=0)
-        mask = tf.cast(mask, tf.float32)
+        mask = tf.cast(mask, attn.dtype)
         attn = tf.reshape(attn, shape=(-1, nw, nb_heads, n, n)) + mask
         attn = tf.reshape(attn, shape=(-1, nb_heads, n, n))
         attn = tf.nn.softmax(attn, axis=-1)
@@ -230,6 +229,7 @@ class SwinTransformerBlock(tf.keras.layers.Layer):
             nb_heads=nb_heads,
             name="attn",
         )
+        self.attn_mask = None
         self.drop_path = DropPath(drop_prob=drop_path_rate)
         self.norm2 = self.norm_layer(name="norm2")
         self.mlp = MLP(
@@ -271,7 +271,7 @@ class SwinTransformerBlock(tf.keras.layers.Layer):
             )
             attn_mask = tf.where(attn_mask != 0, -100.0, attn_mask)
             attn_mask = tf.where(attn_mask == 0, 0.0, attn_mask)
-            # Only the non-trivial attention mask is a Variable, becaue in the PyTorch
+            # Only the non-trivial attention mask is a Variable, because in the PyTorch
             # model, only the non-trivial attention mask exists. The trivial maks
             # is replaced by None and if-statements in the call function.
             self.attn_mask = tf.Variable(
@@ -280,7 +280,9 @@ class SwinTransformerBlock(tf.keras.layers.Layer):
         else:
             # Attention mask is applied additively, so zero-mask has no effect
             # Broadcasting will take care of mapping it to the correct dimensions.
-            self.attn_mask = tf.zeros((1,))
+            self.attn_mask = tf.Variable(
+                initial_value=tf.zeros((1,)), trainable=False, name="attn_mask"
+            )
 
     def call(self, x, training=False):
         window_size = self.window_size
@@ -394,11 +396,15 @@ class SwinTransformerStage(tf.keras.layers.Layer):
         else:
             self.downsample = tf.keras.layers.Activation("linear")
 
-    def call(self, x, training=False):
-        for block in self.blocks:
+    def call(self, x, training=False, return_features=False):
+        features = {}
+        for j, block in enumerate(self.blocks):
             x = block(x, training=training)
+            features[f"block_{j}"] = x
+
         x = self.downsample(x, training=training)
-        return x
+        features["features"] = x
+        return (x, features) if return_features else x
 
 
 @keras_serializable
@@ -417,12 +423,6 @@ class SwinTransformer(tf.keras.Model):
             norm_layer=cfg.norm_layer,
             name="patch_embed",
         )
-        if cfg.ape:  # Absolute position embedding
-            self.absolute_pos_embed = self.add_weight(
-                name="absolute_pos_embed",
-                shape=(1, cfg.nb_patches, cfg.embed_dim),
-                initializer=tf.initializers.Zeros(),
-            )
         self.drop = tf.keras.layers.Dropout(cfg.drop_rate)
 
         # Stochastic depth
@@ -461,24 +461,59 @@ class SwinTransformer(tf.keras.Model):
 
     @property
     def dummy_inputs(self) -> tf.Tensor:
-        return tf.zeros((1, *self.cfg.input_size, self.cfg.in_chans))
+        return tf.zeros((1, *self.cfg.input_size, self.cfg.in_channels))
 
-    def forward_features(self, x, training=False):
+    @property
+    def feature_names(self) -> List[str]:
+        names = ["patch_embedding"]
+        k = 0
+        for j in range(len(self.cfg.nb_blocks)):
+            for _ in range(self.cfg.nb_blocks[j]):
+                names.append(f"block_{k}")
+                k += 1
+            names.append(f"stage_{j}")
+        names += ["features_all", "features", "logits"]
+        return names
+
+    @property
+    def keys_to_ignore_on_load_missing(self) -> List[str]:
+        names = []
+        for j in range(len(self.cfg.nb_blocks)):
+            for k in range(self.cfg.nb_blocks[j]):
+                names.append(f"layers/{j}/blocks/{k}/attn_mask")
+                names.append(f"layers/{j}/blocks/{k}/attn/relative_position_index")
+        return names
+
+    def forward_features(self, x, training=False, return_features=False):
+        features = {}
         x = self.patch_embed(x, training=training)
-        if self.cfg.ape:
-            x = x + self.absolute_pos_embed
         x = self.drop(x, training=training)
+        features["patch_embedding"] = x
+
+        block_idx = 0
+        for stage_idx, stage in enumerate(self.stages):
+            x = stage(x, training=training, return_features=return_features)
+            if return_features:
+                x, stage_features = x
+                for k in range(self.cfg.nb_blocks[stage_idx]):
+                    features[f"block_{block_idx}"] = stage_features[f"block_{k}"]
+                    block_idx += 1
+                features[f"stage_{stage_idx}"] = stage_features["features"]
 
-        for stage in self.stages:
-            x = stage(x, training=training)
         x = self.norm(x, training=training)
+        features["features_all"] = x
         x = self.pool(x)
-        return x
+        features["features"] = x
+        return (x, features) if return_features else x
 
-    def call(self, x, training=False):
-        x = self.forward_features(x, training=training)
+    def call(self, x, training=False, return_features=False):
+        features = {}
+        x = self.forward_features(x, training, return_features)
+        if return_features:
+            x, features = x
         x = self.head(x)
-        return x
+        features["logits"] = x
+        return (x, features) if return_features else x
 
 
 @register_model
